"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExperimentsConfiguration = void 0;
const node_core_library_1 = require("@rushstack/node-core-library");
const terminal_1 = require("@rushstack/terminal");
const experiments_schema_json_1 = __importDefault(require("../schemas/experiments.schema.json"));
const GRADUATED_EXPERIMENTS = new Set(['phasedCommands']);
const _EXPERIMENTS_JSON_SCHEMA = node_core_library_1.JsonSchema.fromLoadedObject(experiments_schema_json_1.default);
/**
 * Use this class to load the "common/config/rush/experiments.json" config file.
 * This file allows repo maintainers to enable and disable experimental Rush features.
 * @public
 */
class ExperimentsConfiguration {
    /**
     * @internal
     */
    constructor(jsonFilePath) {
        try {
            this.configuration = node_core_library_1.JsonFile.loadAndValidate(jsonFilePath, _EXPERIMENTS_JSON_SCHEMA);
        }
        catch (e) {
            if (node_core_library_1.FileSystem.isNotExistError(e)) {
                this.configuration = {};
            }
            else {
                throw e;
            }
        }
        for (const experimentName of Object.getOwnPropertyNames(this.configuration)) {
            if (GRADUATED_EXPERIMENTS.has(experimentName)) {
                // eslint-disable-next-line no-console
                console.log(terminal_1.Colorize.yellow(`The experiment "${experimentName}" has graduated to a standard feature. Remove this experiment from ` +
                    `"${jsonFilePath}".`));
            }
        }
    }
}
exports.ExperimentsConfiguration = ExperimentsConfiguration;
//# sourceMappingURL=ExperimentsConfiguration.js.map