import { CommandLineParameterKind } from '@rushstack/ts-command-line';
/**
 * Information about the available parameters associated with a Rush action
 *
 * @beta
 */
export interface IRushCommandLineParameter {
    /**
     * The corresponding string representation of CliParameterKind
     */
    readonly kind: keyof typeof CommandLineParameterKind;
    /**
     * The long name of the flag including double dashes, e.g. "--do-something"
     */
    readonly longName: string;
    /**
     * An optional short name for the flag including the dash, e.g. "-d"
     */
    readonly shortName?: string;
    /**
     * Documentation for the parameter that will be shown when invoking the tool with "--help"
     */
    readonly description: string;
    /**
     * If true, then an error occurs if the parameter was not included on the command-line.
     */
    readonly required?: boolean;
    /**
     * If provided, this parameter can also be provided by an environment variable with the specified name.
     */
    readonly environmentVariable?: string;
}
/**
 * The full spec of an available Rush command line action
 *
 * @beta
 */
export interface IRushCommandLineAction {
    actionName: string;
    parameters: IRushCommandLineParameter[];
}
/**
 * The full spec of a Rush CLI
 *
 * @beta
 */
export interface IRushCommandLineSpec {
    actions: IRushCommandLineAction[];
}
/**
 * Information about the available CLI commands
 *
 * @beta
 */
export declare class RushCommandLine {
    static getCliSpec(rushJsonFolder: string): IRushCommandLineSpec;
}
//# sourceMappingURL=RushCommandLine.d.ts.map