"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.RushCommandLine = void 0;
const ts_command_line_1 = require("@rushstack/ts-command-line");
const RushCommandLineParser_1 = require("../cli/RushCommandLineParser");
const _commandLineSpecByWorkspaceFolder = new Map();
/**
 * Information about the available CLI commands
 *
 * @beta
 */
class RushCommandLine {
    static getCliSpec(rushJsonFolder) {
        let result = _commandLineSpecByWorkspaceFolder.get(rushJsonFolder);
        if (!result) {
            const commandLineParser = new RushCommandLineParser_1.RushCommandLineParser({ cwd: rushJsonFolder });
            // extract the set of command line elements from the command line parser
            const actions = [];
            for (const { actionName, parameters: rawParameters } of commandLineParser.actions) {
                const parameters = [];
                for (const { kind: rawKind, longName, shortName, description, required, environmentVariable } of rawParameters) {
                    parameters.push({
                        kind: ts_command_line_1.CommandLineParameterKind[rawKind],
                        longName,
                        shortName,
                        description,
                        required,
                        environmentVariable
                    });
                }
                actions.push({
                    actionName,
                    parameters
                });
            }
            result = { actions };
            _commandLineSpecByWorkspaceFolder.set(rushJsonFolder, result);
        }
        return result;
    }
}
exports.RushCommandLine = RushCommandLine;
//# sourceMappingURL=RushCommandLine.js.map