"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.RushInternals = void 0;
const Rush_1 = require("./Rush");
/**
 * Used by rush-sdk to access internals of rush-lib.
 * @internal
 */
class RushInternals {
    /**
     * Used by rush-sdk to load an internal API specified by its module path.
     *
     * @param srcImportPath - The module path to load.  For example, to refer to `src/api/ChangeFile.ts`,
     * the `srcImportPath` would be `"api/ChangeFile"`.
     * @returns the module object as would be returned by `require()`
     */
    static loadModule(srcImportPath) {
        const libPath = `${Rush_1.Rush._rushLibPackageFolder}/lib/${srcImportPath}`;
        try {
            return require(libPath);
        }
        catch (e) {
            throw new Error(`The specified internal API "src/${srcImportPath}" is not implemented by Rush ${Rush_1.Rush.version}`);
        }
    }
}
exports.RushInternals = RushInternals;
//# sourceMappingURL=RushInternals.js.map