import type { RushConfigurationProject } from '../../api/RushConfigurationProject';
import type { BaseShrinkwrapFile } from './BaseShrinkwrapFile';
/**
 * This class handles creating the project/.rush/temp/shrinkwrap-deps.json file
 * which tracks the direct and indirect dependencies that a project consumes. This is used
 * to better determine which projects should be rebuilt when dependencies are updated.
 */
export declare abstract class BaseProjectShrinkwrapFile<TShrinkwrapFile extends BaseShrinkwrapFile> {
    readonly projectShrinkwrapFilePath: string;
    protected readonly project: RushConfigurationProject;
    /**
     * The shrinkwrap file that the project shrinkwrap file is based off of.
     */
    protected readonly shrinkwrapFile: TShrinkwrapFile;
    constructor(shrinkwrapFile: TShrinkwrapFile, project: RushConfigurationProject);
    /**
     * Save an empty project shrinkwrap file. This is used in repos with no dependencies.
     */
    static saveEmptyProjectShrinkwrapFileAsync(project: RushConfigurationProject): Promise<void>;
    /**
     * Get the fully-qualified path to the <project>/.rush/temp/shrinkwrap-deps.json
     * for the specified project.
     */
    static getFilePathForProject(project: RushConfigurationProject): string;
    /**
     * If the <project>/.rush/temp/shrinkwrap-deps.json file exists, delete it. Otherwise, do nothing.
     */
    deleteIfExistsAsync(): Promise<void>;
    /**
     * Generate and write the project shrinkwrap file to <project>/.rush/temp/shrinkwrap-deps.json.
     *
     * @virtual
     */
    abstract updateProjectShrinkwrapAsync(): Promise<void>;
}
//# sourceMappingURL=BaseProjectShrinkwrapFile.d.ts.map