"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.NamedProjectSelectorParser = void 0;
const node_core_library_1 = require("@rushstack/node-core-library");
const RushConstants_1 = require("../RushConstants");
class NamedProjectSelectorParser {
    constructor(rushConfiguration) {
        this._rushConfiguration = rushConfiguration;
    }
    async evaluateSelectorAsync({ unscopedSelector, terminal, parameterName }) {
        const project = this._rushConfiguration.findProjectByShorthandName(unscopedSelector);
        if (!project) {
            terminal.writeErrorLine(`The project name "${unscopedSelector}" passed to "${parameterName}" does not exist in ` +
                `${RushConstants_1.RushConstants.rushJsonFilename}.`);
            throw new node_core_library_1.AlreadyReportedError();
        }
        return [project];
    }
    getCompletions() {
        const unscopedNamesMap = new Map();
        const scopedNames = new Set();
        for (const project of this._rushConfiguration.rushConfigurationJson.projects) {
            scopedNames.add(project.packageName);
            const unscopedName = node_core_library_1.PackageName.getUnscopedName(project.packageName);
            const count = unscopedNamesMap.get(unscopedName) || 0;
            unscopedNamesMap.set(unscopedName, count + 1);
        }
        const unscopedNames = [];
        for (const [unscopedName, unscopedNameCount] of unscopedNamesMap) {
            // don't suggest ambiguous unscoped names
            if (unscopedNameCount === 1 && !scopedNames.has(unscopedName)) {
                unscopedNames.push(unscopedName);
            }
        }
        return unscopedNames.sort().concat([...scopedNames].sort());
    }
}
exports.NamedProjectSelectorParser = NamedProjectSelectorParser;
//# sourceMappingURL=NamedProjectSelectorParser.js.map