import { type ITerminalProvider, Terminal } from '@rushstack/terminal';
/**
 * @beta
 */
export interface ILogger {
    readonly terminal: Terminal;
    /**
     * Call this function to emit an error to the Rush runtime.
     */
    emitError(error: Error): void;
    /**
     * Call this function to emit a warning to the Rush runtime.
     */
    emitWarning(warning: Error): void;
}
export interface ILoggerOptions {
    loggerName: string;
    terminalProvider: ITerminalProvider;
    getShouldPrintStacks: () => boolean;
}
export declare class Logger implements ILogger {
    private readonly _options;
    private readonly _errors;
    private readonly _warnings;
    readonly terminal: Terminal;
    constructor(options: ILoggerOptions);
    get errors(): ReadonlyArray<Error>;
    get warnings(): ReadonlyArray<Error>;
    static getErrorMessage(error: Error): string;
    /**
     * {@inheritdoc ILogger.emitError}
     */
    emitError(error: Error): void;
    /**
     * {@inheritdoc ILogger.emitWarning}
     */
    emitWarning(warning: Error): void;
    private get _shouldPrintStacks();
}
//# sourceMappingURL=Logger.d.ts.map