// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
import { CommandLineParameterKind } from '@rushstack/ts-command-line';
import { RushCommandLineParser } from '../cli/RushCommandLineParser';
const _commandLineSpecByWorkspaceFolder = new Map();
/**
 * Information about the available CLI commands
 *
 * @beta
 */
export class RushCommandLine {
    static getCliSpec(rushJsonFolder) {
        let result = _commandLineSpecByWorkspaceFolder.get(rushJsonFolder);
        if (!result) {
            const commandLineParser = new RushCommandLineParser({ cwd: rushJsonFolder });
            // extract the set of command line elements from the command line parser
            const actions = [];
            for (const { actionName, parameters: rawParameters } of commandLineParser.actions) {
                const parameters = [];
                for (const { kind: rawKind, longName, shortName, description, required, environmentVariable } of rawParameters) {
                    parameters.push({
                        kind: CommandLineParameterKind[rawKind],
                        longName,
                        shortName,
                        description,
                        required,
                        environmentVariable
                    });
                }
                actions.push({
                    actionName,
                    parameters
                });
            }
            result = { actions };
            _commandLineSpecByWorkspaceFolder.set(rushJsonFolder, result);
        }
        return result;
    }
}
//# sourceMappingURL=RushCommandLine.js.map