import { type JsonObject } from '@rushstack/node-core-library';
/**
 * A base class for flag file.
 * @internal
 */
export declare class FlagFile<TState extends JsonObject = JsonObject> {
    /**
     * Flag file path
     */
    readonly path: string;
    /**
     * Content of the flag
     */
    protected _state: TState;
    /**
     * Creates a new flag file
     * @param folderPath - the folder that this flag is managing
     * @param state - optional, the state that should be managed or compared
     */
    constructor(folderPath: string, flagName: string, initialState: TState);
    /**
     * Returns true if the file exists and the contents match the current state.
     */
    isValidAsync(): Promise<boolean>;
    /**
     * Writes the flag file to disk with the current state
     */
    createAsync(): Promise<void>;
    /**
     * Removes the flag file
     */
    clearAsync(): Promise<void>;
}
//# sourceMappingURL=FlagFile.d.ts.map