/**
 * @internal
 */
export interface IOperationStateFileOptions {
    projectFolder: string;
    metadataFolder: string;
}
/**
 * @internal
 */
export interface IOperationStateJson {
    nonCachedDurationMs: number;
    cobuildContextId: string | undefined;
    cobuildRunnerId: string | undefined;
}
/**
 * A helper class for managing the state file of a operation.
 *
 * @internal
 */
export declare class OperationStateFile {
    private _state;
    /**
     * The path of the state json file.
     *
     * Example: `/code/repo/my-project/.rush/temp/operation/_phase_build/state.json`
     */
    readonly filepath: string;
    /**
     * The relative path of the state json file to project folder
     *
     * Example: `.rush/temp/operation/_phase_build/state.json`
     */
    readonly relativeFilepath: string;
    static filename: string;
    constructor(options: IOperationStateFileOptions);
    get state(): IOperationStateJson | undefined;
    writeAsync(json: IOperationStateJson): Promise<void>;
    tryRestoreAsync(): Promise<IOperationStateJson | undefined>;
}
//# sourceMappingURL=OperationStateFile.d.ts.map