import { type ITerminal } from '@rushstack/terminal';
import type { RushConfiguration } from '../../api/RushConfiguration';
import type { VersionMismatchFinderEntity } from './VersionMismatchFinderEntity';
import type { Subspace } from '../../api/Subspace';
export interface IVersionMismatchFinderOptions {
    subspace?: Subspace;
    variant: string | undefined;
}
export interface IVersionMismatchFinderRushCheckOptions extends IVersionMismatchFinderOptions {
    printAsJson?: boolean | undefined;
    truncateLongPackageNameLists?: boolean | undefined;
}
export interface IVersionMismatchFinderEnsureConsistentVersionsOptions extends IVersionMismatchFinderOptions {
}
export interface IVersionMismatchFinderGetMismatchesOptions extends IVersionMismatchFinderOptions {
}
export interface IMismatchDependency {
    dependencyName: string;
    versions: IMismatchDependencyVersion[];
}
export interface IMismatchDependencyVersion {
    version: string;
    projects: string[];
}
export interface IMismatchDependencies {
    mismatchedVersions: IMismatchDependency[];
}
export declare class VersionMismatchFinder {
    private _allowedAlternativeVersion;
    private _mismatches;
    private _projects;
    constructor(projects: VersionMismatchFinderEntity[], allowedAlternativeVersions?: Map<string, ReadonlyArray<string>>);
    static rushCheck(rushConfiguration: RushConfiguration, terminal: ITerminal, options?: IVersionMismatchFinderRushCheckOptions): void;
    static ensureConsistentVersions(rushConfiguration: RushConfiguration, terminal: ITerminal, options?: IVersionMismatchFinderEnsureConsistentVersionsOptions): void;
    /**
     * Populates a version mismatch finder object given a Rush Configuration.
     * Intentionally considers preferred versions.
     */
    static getMismatches(rushConfiguration: RushConfiguration, options?: IVersionMismatchFinderOptions): VersionMismatchFinder;
    private static _checkForInconsistentVersions;
    get mismatches(): ReadonlyMap<string, ReadonlyMap<string, readonly VersionMismatchFinderEntity[]>>;
    get numberOfMismatches(): number;
    getMismatches(): string[];
    getVersionsOfMismatch(mismatch: string): string[] | undefined;
    getConsumersOfMismatch(mismatch: string, version: string): VersionMismatchFinderEntity[] | undefined;
    printAsJson(): void;
    print(truncateLongPackageNameLists?: boolean): void;
    private _analyze;
    private _isVersionAllowedAlternative;
    private _getKeys;
}
//# sourceMappingURL=VersionMismatchFinder.d.ts.map